#!/usr/bin/env python3

"""Chained CI config wrapper."""

import sys
from schema import Schema, Or, Optional
from chainedci.tools import raise_ex
from chainedci.log import log
from chainedci.step import Step
from chainedci.config import config, getConfig


def _run_status_exit_signal(status):
    """Select the exit signal depending on status."""
    switcher = {
        'critical': 5,
        'failed': 3,
        'canceled': 2,
        'skipped': 1
    }
    return switcher.get(status, 0)


class Scenario(dict):
    """Scenario Object."""

    def __init__(self, name):
        """
        Prepare the scenario object.

        :param name: The name of the scenario
        :type key: string
        """
        super().__init__()
        self.name = name
        self.steps = {}
        log.info("Scenario[%s]: initialization", self.name)
        if name in config['scenarios']:
            self.update(self._validate_sc(config['scenarios'][name]))
            config_run = getConfig('run')
            if 'job' in config_run:
                step_name = config_run['job']
                self.load_step(step_name, self['scenario_steps'][step_name])
            else:
                for step_name, step_config in self['scenario_steps'].items():
                    self.load_step(step_name, step_config)
        else:
            raise_ex(AttributeError, "No such scenario: " + name)

    def load_step(self, step_name, step_config):
        """Add a new step in the Scenario."""
        default = self._validate_all(config['defaults'])
        log.debug("Scenario[%s]: loading step '%s'", self.name,
                  step_name)
        cfg = default['gitlab']['git_projects'][step_config['project']]
        cfg.update(step_config)
        if 'ssh_access' in default:
            log.debug("Scenario[%s]: push group ssh_access '%s'", self.name,
                      step_name)
            cfg.update(
                {'group_ssh_access': default['ssh_access']})
        if 'ansible_ssh_creds' in default:
            log.debug("Scenario[%s]: push group ssh_creds '%s'", self.name,
                      step_name)
            cfg.update(
                {'group_ssh_creds': default['ansible_ssh_creds']})
        if 'certificates' in default:
            log.debug("Scenario[%s]: push group certificates '%s'", self.name,
                      step_name)
            cfg.update(
                {'group_certificates': default['certificates']})
        extra = {'pod': self.name, 'jumphost': self['jumphost']}
        self.steps[step_name] = Step(step_name, cfg, extra, self.name)

    def run(self, step_name):
        """
        Run the step pipeline.

        :param step_name: The name of the step
        :type step_name: string
        """
        if step_name not in self.steps:
            raise_ex(ValueError, f"Scenario - Step name '{step_name}' unknown")
        try:
            run_status = self.steps[step_name].run()
            log.info("Scenario[%s]: run finished wth status '%s'",
                     self.name, run_status)
        # pylint: disable=broad-except
        except Exception as err:
            log.critical("Here is the exception:\n %s", err)
            log.critical("Scenario[%s]: Run crash, please check previous "
                         "message", self.name)
            run_status = 'critical'
        sys.exit(_run_status_exit_signal(run_status))

    def _validate_all(self, src):
        log.debug("Scenario[%s]: validation of global config", self.name)
        src = dict(src)
        schema = Schema({
            Optional('disable_pages'): bool,
            Optional('protected_pods'): [str],
            Optional('tokens'): [
                {
                    'name': str,
                    'filter': str,
                    'id': str,
                    'value': str}
            ],
            'stages': [str],
            'runner': {
                'tags': [str],
                'env_vars': Or(dict, None),
                Optional('docker_proxy'): Or(str, None),
                'images': dict,
            },
            'gitlab': {
                Optional('pipeline'): {'delay': int},
                'base_url': str,
                'api_url': str,
                'private_token': str,
                Optional('healthchecks_url'): str,
                'git_projects': dict
            },
            Optional('certificates'): str,
            Optional('ansible_ssh_creds'): str,
            Optional('ssh_access'): str,
        })
        return schema.validate(src)

    def _validate_sc(self, src):
        log.debug("Scenario[%s]: validation of scenario", self.name)
        schema = Schema({
            'jumphost': Or({
                'server': str,
                'user': str}, None, {}),
            Optional('environment'): str,
            'scenario_steps': dict})
        return schema.validate(src)



#!/usr/bin/env python3

"""Chained CI Scenario Step."""

from schema import Schema, Or, Optional
from ansible.parsing.yaml.objects import AnsibleVaultEncryptedUnicode
from chainedci.config import config, getConfig, merge_dicts
from chainedci.parser import JinjaParser
from chainedci.log import log, deprecation_warning
from chainedci.artifact import Artifact
from chainedci.artifact_src import ArtifactSrc
from chainedci.pipeline import Pipeline


def _get_config_step():
    """Get config step config if exists."""
    ini = getConfig('ini')
    config_step_name = ini['specific_steps']['config']
    scenario_name = getConfig('run')['name']
    if config_step_name in config[scenario_name]['scenario_steps']:
        return config[scenario_name]['scenario_steps'][config_step_name]
    return None


def _select_values_from_config_step(config_project):
    """
    Select values from config_project.

    Return project api (or url) and config path if exists
    """
    config_src = {}
    if 'api' in config_project:
        config_src['api'] = config_project['api']
    elif 'url' in config_project:
        config_src['url'] = config_project['url']
    if 'path' in config_project:
        config_src['path'] = config_project['path']
    return config_src


class Step(dict):
    """Scenario step object."""

    def __init__(self, name, step_config, extra, scenario):
        """
        Prepare the step object.

        :param name: The name of the step
        :type name: string
        :param step_config: The step config
        :type step_config: dict
        :param extra: Extra parameters to send to the remote pipeline
        :type extra: dict
        :param scenario: Scenario name
        :type scenario: string
        :return: a Step object
        :rtype: Step class
        """
        super().__init__()
        self.name = name
        self.extra = extra
        self.pipeline = None
        self.scenario = scenario
        self.artifact = None
        self.artifact_input = {'encrypt': False}
        log.info("Step[%s]: initialization", self.name)
        self.update(self._validate(step_config))
        self._prepare_artifact()
        self._prepare_parameters()

    def run(self):
        """
        Run the pipeline.

        :return: Run result
        :rtype: string

        Depending on step name,
        Get the artifact if it is a config step
        OR run another chainedci pipeline
        OR call the remote pipeline trigger
        """
        return (self._run_skip()
                or self._run_config()
                or self._run_trigger_myself()
                or self._run_project())

    def _run_skip(self):
        log.debug("Step[%s]: check if we run this step", self.name)
        if not self.only_except_check():
            log.info("Step[%s]: Skip due to only/except reason")
            return "skipped"
        return None

    def _run_config(self):
        """Run a config step.

        :return: Run result
        :rtype: string
        """
        ini = getConfig('ini')
        if self.name == ini['specific_steps']['config']:
            log.info("Step[%s]: run - Specific step 'Config'", self.name)
            if self.artifact:
                self.artifact.get_local()
                return "ok"
            log.error("Step[%s]: run - no artifact, nothing to do", self.name)
            return "skipped"
        return None

    def _run_trigger_myself(self):
        """Run a trigger to the same chained-ci.

        :return: Run result
        :rtype: string
        """
        ini = getConfig('ini')
        if self.name == ini['specific_steps']['trigger_myself']:
            parser = JinjaParser()
            log.info("Step[%s]: run - Specific step 'Trigger'",
                     self.name)
            self.extra['INPOD'] = self.extra['pod']
            self.extra['triggered_from'] = ini['env']['CI_JOB_NAME']
            self['api'] = (f"{config['gitlab']['api_url']}"
                           f"/projects/{ini['env']['CI_PROJECT_ID']}"
                           "/trigger/pipeline")
            self.pipeline = Pipeline(self['api'],
                                     self['trigger_token'],
                                     self.parameters,
                                     (parser.parse(self['branch'])
                                      if ('branch' in self) else 'master')
                                     )
            self.pipeline.start()
            log.info("Step[%s]: trigger started", self.name)
            return self.pipeline.status['status']
        return None

    def _run_project(self):
        """Run a project pipeline.

        :return: Run result
        :rtype: string
        """
        parser = JinjaParser()
        if self.artifact:
            self.parameters['artifacts_bin'] = self.artifact.bin
        self.pipeline = Pipeline(self['api'],
                                 self['trigger_token'],
                                 self.parameters,
                                 (parser.parse(self['branch'])
                                  if ('branch' in self) else 'master')
                                 )
        if self.get('timeout', None):
            log.debug("Step[%s]: set pipeline timeout to custom value"
                      " '%s'", self.name, self['timeout'])
            self.pipeline.set_custom_timeout(self['timeout'])
        self.pipeline.loop()
        if self.get('pull_artifacts', False):
            self._pull_artifact()
        log.info("Step[%s]: run finished", self.name)
        return self.pipeline.status['status']

    def only_except_check(self):
        """
        Check if run conditions are present.

        Eval each "only" and "except" conditions on env vars and return the
        result with formula: any(except[]) and all(only[])
        """
        ini = getConfig('ini')
        if 'only' not in self and 'except' not in self:
            log.debug("Step[%s]: No only/except rules", self.name)
            return True
        # pylint: disable=R1729
        excpt = (any([ini['env'].eval(rule)
                      for rule in self['except']])
                 if 'except' in self else False)
        # pylint: disable=R1729
        only = (all([ini['env'].eval(rule)
                     for rule in self['only']])
                if 'only' in self else True)
        return not excpt and only

    def _prepare_artifact(self):
        """Prepare the artifact to send."""
        log.debug("Step[%s]: prepare input artifact", self.name)
        if 'input_artifact' in list(self.keys()):
            self.artifact_input = self['input_artifact']
        else:
            deprecation_warning(f"please update step '{self.name}' with new "
                                "'input_artifact' structure")
            self._prepare_artifact_deprecated()
        self._prepare_artificat_input_security_values()
        self._prepare_artifact_from_input()

    def _prepare_artificat_input_security_values(self):
        """Prepare security fields linked to security."""
        self._prepare_artificat_input_encrypt()
        self._prepare_artificat_input_group_ssh_access()
        self._prepare_artificat_input_group_ssh_creds()
        self._prepare_artificat_input_group_certificates()

    def _prepare_artificat_input_encrypt(self):
        """Set default 'encrypt' value in artifact input."""
        if ('encrypt' not in self.artifact_input
                and self.artifact_input):
            log.debug("Step[%s]: option: do not encrypt artifact", self.name)
            self.artifact_input['encrypt'] = False

    def _prepare_artificat_input_group_ssh_access(self):
        """Inherit ssh_access from group."""
        if ('group_ssh_access' in self
                and 'ssh_access' not in self.artifact_input):
            log.debug("Step[%s]: option: use group ssh access '%s'",
                      self.name, self['group_ssh_access'])
            self.artifact_input['ssh_access'] = self['group_ssh_access']

    def _prepare_artificat_input_group_ssh_creds(self):
        """Inherit ssh_access from group."""
        if ('group_ssh_creds' in self
                and 'ssh_creds' not in self.artifact_input):
            log.debug("Step[%s]: option: use group ssh_creds '%s'",
                      self.name, self['group_ssh_creds'])
            self.artifact_input['ssh_creds'] = self['group_ssh_creds']

    def _prepare_artificat_input_group_certificates(self):
        """Inherit certificates from group."""
        if ('group_certificates' in self
                and 'certificates' not in self.artifact_input):
            log.debug("Step[%s]: option: use group certificate '%s'",
                      self.name, self['group_certificates'])
            self.artifact_input['certificates'] = self['group_certificates']

    def _prepare_artifact_from_input(self):
        """Prepare the artifact object."""
        if self.artifact_input:
            config_step = _get_config_step()
            if config_step:
                config_src = self._enrich_config_src_with_config_step()
            else:
                config_src = {'branch':
                              self._prepare_artifact_default_branch()}
            self.artifact = Artifact(
                self.artifact_input, self.name, config_src)
        else:
            self.artifact = None

    def _enrich_config_src_with_config_step(self):
        """Enrich artifact input src from 'config' step.

        If a config step is in the scenario, we reuse the config to
        fetch the artifacts.
        """
        config_src = {}
        config_step = _get_config_step()
        log.debug("Step[%s]: prepare config_src", self.name)
        config_project_name = config_step['project']
        config_project = config['defaults']['gitlab']['git_projects'][
            config_project_name]
        config_project = merge_dicts(config_project, config_step)
        config_src = _select_values_from_config_step(config_project)
        # Always set the branch, from config step branch or from
        # env vars
        ini = getConfig('ini')
        config_src['branch'] = \
            config_project.get('branch',
                               ini['env'].get('CI_COMMIT_BRANCH',
                                              'master'))
        return config_src

    def _prepare_artifact_default_branch(self):
        """Set a default branch to artifact src."""
        ini = getConfig('ini')
        return self.get('branch',
                        ini['env'].get('CI_COMMIT_BRANCH',
                                       'master'))

    def _prepare_artifact_deprecated(self):
        """Prepare artifact from deprecated inputs."""
        self.artifact_input = {}
        self._prepare_artifact_deprecated_files()
        self._prepare_artifact_deprecated_infra()
        self._prepare_artifact_deprecated_secu()

    def _prepare_artifact_deprecated_files(self):
        if 'get_artifacts' in self:
            self.artifact_input['get_artifacts'] = self['get_artifacts']
        if 'local_files' in self:
            self.artifact_input['local_files'] = self['local_files']
        if 'remote_files' in self:
            self.artifact_input['remote_files'] = self['remote_files']

    def _prepare_artifact_deprecated_infra(self):
        ini = getConfig('ini')
        if self.name == ini['specific_steps']['config']:
            if self.get('infra', '').lower() == 'none':
                log.debug("Step[%s]: No infra file added", self.name)
            else:
                self.artifact_input['infra_pdfidf'] = self.get('infra',
                                                               self.scenario)

    def _prepare_artifact_deprecated_secu(self):
        if 'certificates' in self:
            self.artifact_input['certificates'] = self['certificates']
        if 'ssh_access' in self:
            self.artifact_input['ssh_access'] = self['ssh_access']
        if 'ansible_ssh_creds' in self:
            self.artifact_input['ssh_creds'] = self['ansible_ssh_creds']
        if 'get_encrypt' in self:
            self.artifact_input['encrypt'] = self['get_encrypt']

    def _prepare_parameters(self):
        """Prepare the parameters to send."""
        log.debug("Step[%s]: prepare pipeline parameters", self.name)
        params = self._prepare_parameters_get_and_parse('parameters')
        params |= self._prepare_parameters_get_and_parse('extra_parameters')
        if 'novault' in self:
            params['NOVAULT_LIST'] = '\\n'.join(self['novault'])
        params['source_job_name'] = self.name
        for p_name, p_value in self.extra.items():
            params[p_name] = p_value
        self.parameters = params

    def _prepare_parameters_get_and_parse(self, param_type):
        """Get and parse params."""
        params = {}
        parser = JinjaParser()
        if param_type in self:
            for k, val in self[param_type].items():
                params[k] = parser.parse(val)
        return params

    def _pull_artifact(self):
        """Download the project pipeline artifact."""
        url = self.pipeline.artifact_url(self['pull_artifacts'])
        artifact_src = ArtifactSrc({'type': 'remote_archive', 'source': url})
        artifact_src.unpack()

    def _validate(self, src):
        src = dict(src)
        log.debug("Step[%s]: validation", self.name)
        schema = Schema({'project': str,
                         'stage': str,
                         Optional('url'): str,
                         Optional('api'): str,
                         Optional('path'): str,
                         Optional('token'): str,
                         Optional('infra'): str,
                         Optional('branch'): str,
                         Optional('novault'): [str],
                         Optional('get_encrypt'): bool,
                         Optional('get_bin'): bool,
                         Optional('input_artifact'): dict,
                         Optional('get_artifacts'): Or(str, list),
                         Optional('only'): [str],
                         Optional('except'): [str],
                         Optional('local_files'): [dict],
                         Optional('remote_files'): [dict],
                         Optional('certificates'): str,
                         Optional('group_certificates'): str,
                         Optional('ansible_ssh_creds'): str,
                         Optional('group_ssh_creds'): str,
                         Optional('ssh_access'): str,
                         Optional('group_ssh_access'): str,
                         Optional('parameters'): dict,
                         Optional('extra_parameters'): dict,
                         Optional('pull_artifacts'): Or(str, None),
                         Optional('trigger_token'): Or(
                             str, AnsibleVaultEncryptedUnicode),
                         Optional('timeout'): int})
        return schema.validate(src)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright 2019 Orange
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Options module."""


import argparse
import textwrap
from os import path
from chainedci.log import log, deprecation_warning
from chainedci.tools import raise_ex
from chainedci.config import getConfig
from chainedci.version import __version__


class Options():
    """Get options from args."""

    def __init__(self):
        """Init Inventory."""
        self.get_options()
        self.check_options()
        self.check_required_parameters()

    def get_options(self):
        """Define and parse the command line arguments."""
        description = textwrap.dedent(f'''\
            chainedci {__version__}
            ---------------------------

                Mode: 'run'
                Prepare and trigger a specific step of a chainedci scenario.
                Parameters are backward compatible with legacy ansible version.
                Required parameters: '--scenario', '--job', '--inventory'

                Mode: 'generate'
                Prepare a .gitlab-ci.yml file from inventory
                Required parameters: '--inventory'

                Mode: 'init'
                Init the chained-ci project structure
                Required parameters: None

            ''')
        parser = argparse.ArgumentParser(
            description=description,
            formatter_class=argparse.RawDescriptionHelpFormatter)

        parser.add_argument('mode',
                            help="mode in 'run', 'generate'",
                            choices=['run', 'generate', 'init'])
        parser.add_argument('-i', '--inventory',
                            dest='inventory',
                            help="inventory of scenarios")
        parser.add_argument('-s', '--scenario', '-l', '--limit',
                            dest='scenario_name',
                            help="limit inventory to one scenario")
        parser.add_argument('-j', '--job',
                            dest='job',
                            help="job name to execute")
        parser.add_argument('-p', '--vault-password-file',
                            help="path to the ansible vault password file",
                            dest='vault_password_file')
        parser.add_argument('-e', '--extra_vars',
                            help="backward compatible argument, please use "
                                 "--job parameter. This parameter must be "
                                 "formatted like this: 'step=xxxxx'")
        self.opts = parser.parse_args()

    def check_options(self):
        """
        Check all arguments.

        - ensure retro-compatibility
        - check file presence
        """
        # Check job parameter in case of retrocompatibility
        if self.opts.mode == 'run':
            if not self.opts.job:
                if 'step=' not in (self.opts.extra_vars or ''):
                    raise_ex(ValueError, "Please set --job parameter or "
                             "--extra_vars parameter with "
                             "'step=xxxx' "
                             "value")
                else:
                    extra = self.opts.extra_vars.split("=")
                    self.opts.job = extra[1]
                    deprecation_warning("Please consider using option --job "
                                        "to replace '-e step=xxx' parameter")
        # store vault key if exists
        self.store_vault_key()
        # crash if inventory does not exists in run or generate mode
        if (self.opts.mode in ['generate', 'run'] and
                not path.exists(self.opts.inventory)):
            raise_ex(ValueError, f"Inventory file '{self.opts.inventory}' "
                                 "does not exists.")

    def store_vault_key(self):
        """Store vault key in 'run' config."""
        ini = getConfig('ini')
        run = getConfig('run')
        if self.opts.vault_password_file:
            if not path.exists(self.opts.vault_password_file):
                raise_ex(ValueError, "Vault password file "
                                     f"'{self.opts.vault_password_file}' "
                                     "does not exists.")
            with open(self.opts.vault_password_file,
                      encoding="utf-8") as vault_file:
                content = vault_file.readlines()
                run['key'] = content[0].rstrip().encode("utf-8")
                log.warning("Vault password set with file %s",
                            self.opts.vault_password_file)
        elif ini['encryption']['key_env_name'] in ini['env']:
            log.warning("Vault password set with ENV VAR")
            run['key'] = ini['env'][ini['encryption']
                                    ['key_env_name']].encode("utf-8")
        else:
            log.warning("No vault password is set, assume all the "
                        "configuration is readable without ansible vault")

    def check_required_parameters(self):
        """Check parameters depending on 'mode'."""
        if self.opts.mode == 'run':
            if (not self.opts.job
                    or not self.opts.scenario_name):
                raise_ex(ValueError, "'Run' mode requires following "
                                     "parameters: '--scenario', "
                                     "'--job', and '--inventory'")



#!/usr/bin/env python3

"""Config and Logging usefull scripts and vars."""

import logging
import sys
from time import sleep
from chainedci.config import getConfig, config
from chainedci.parser import JinjaParser


def raise_ex(exception, message):
    """Raise an exception after logging it."""
    log = logging.getLogger('chainedci')
    log.critical(message)
    ini = getConfig('ini')
    if ini['log']['exception_or_exit']:
        raise exception(message)
    sys.exit(message)


def get_env_or_raise(env):
    """Return a env var or raise an exception."""
    ini = getConfig('ini')
    if env not in ini['env']:
        raise_ex(ValueError, f"Environmnent variable '{env}' MUST be set")
    return ini['env'][env]


def token_selector(url):
    """
    Return a token based on domain name.

    If no domain is found, return none.
    """
    log = logging.getLogger('chainedci')
    if 'tokens' not in config['defaults']:
        log.debug("token_selector - none found")
        return {}
    log.debug("token_selector - check in the %s token(s)",
              len(config['defaults']['tokens']))
    for token in config['defaults']['tokens']:
        log.debug("token_selector - check token '%s' with filter '%s'",
                  token['name'], token['filter'])
        if token['filter'] in url:
            log.debug("token_selector - it match")
            parser = JinjaParser()
            log.debug("token_selector - type of token value: %s",
                      type(str(parser.parse(token['value']))))

            token_name = str(token['id'])
            log.debug("token_selector - type of token key: %s",
                      type(token_name))
            return {token_name: str(parser.parse(token['value']))}
    log.debug("token_selector - none found")
    return {}


def waiting_cursor(sleep_time=15, micro_step=5, end=''):
    """
    Print a waiting cursor.

    :param sleep_time: time wainting in seconds
    :type sleep_time: int
    :param micro_step: time between a dot print in seconds
    :type micro_step: int
    """
    ini = getConfig('ini')
    if ini['log']['level'] == 'INFO':
        for elapsed in range(0, sleep_time):
            print_one_on(elapsed, micro_step)
            sleep(1)
        print('#', end=end, flush=True)
    else:
        sleep(sleep_time)


def print_one_on(index, micro_step, string='.', end=''):
    """
    Print a string only if index can be devided by micro_step.

    :param index: one index
    :type sleep_time: int
    :param micro_step: a step when we can print the string
    :type micro_step: int
    :param string: the string to print
    :type string: string
    :param end: end of line
    :type end: string
    """
    if index % micro_step == 0:
        print(string, end=end, flush=True)



#!/usr/bin/env python3

"""Chained CI Artifact."""
import urllib.parse
import base64
from os import walk
from shutil import rmtree, make_archive
# pylint: disable-next=deprecated-module
from distutils.dir_util import copy_tree
from tempfile import mkdtemp, NamedTemporaryFile
from schema import Schema, Or, Optional
from ansible.parsing.vault import VaultSecret, VaultEditor
from ansible.errors import AnsibleError
from chainedci.config import getConfig
from chainedci.log import log
from chainedci.artifact_src import ArtifactSrc
from chainedci.parser import JinjaParser


class Artifact(dict):
    """
    Object representing the artifact to send to the remote pipeline.

    This object:
    - stores the Artifact sources
    - encrypt all sources
    - generate the zip file
    - generate the bin value of the Artifact
    """

    def __init__(self, src, step_name, config_source):
        """Artifact class.

        :param src: The artifact sources
        :type src: dict
        :param ini: The chainedci ini config
        :type ini: dict
        """
        super().__init__()
        self.sources = []
        self.step_name = step_name
        self.folder = None
        self.vault_ed = None
        self.config_source = config_source
        log.info("Artifact[%s]: initialization", self.step_name)
        self.update(self._validate(src))
        self.import_src()

    def import_src(self):
        """Import artifact sources."""
        ini = getConfig('ini')
        if 'get_artifacts' in self:
            log.debug("Artifact[%s]: load 'get_artifacts'", self.step_name)
            if isinstance(self['get_artifacts'], str):
                self._add_source({'type': 'pipeline',
                                  'source': self['get_artifacts']})
            else:
                for artifact in self['get_artifacts']:
                    if 'static_src' in artifact:
                        (artifact_type, base, final) = self._get_source(
                            extension=ini['artifacts']['static']['extension'],
                            folder=ini['artifacts']['static']['src'])
                        artifact['type'] = f'{artifact_type}_archive'
                        artifact['source'] = (
                            f"{base}/{artifact['name']}{final}")
                        del artifact['static_src']
                    else:
                        artifact['type'] = 'pipeline'
                        artifact['source'] = artifact['name']
                    del artifact['name']
                    self._add_source(artifact)
        if 'local_files' in self:
            log.debug("Artifact[%s]: load 'local_files'", self.step_name)
            for file in self['local_files']:
                (source_file, local_file) = list(file.items())[0]
                self._add_source({'type': 'local',
                                  'source': source_file,
                                  'destination': local_file})
        if 'remote_files' in self:
            log.debug("Artifact[%s]: load 'remote_files'", self.step_name)
            for file in self['remote_files']:
                (source_file, remote_file) = list(file.items())[0]
                self._add_source({'type': 'remote',
                                  'source': source_file,
                                  'destination': remote_file})
        if 'infra_pdfidf' in self:
            log.debug("Artifact[%s]: load 'infra_pdfidf'", self.step_name)
            self._import_infra_pdfidf(self['infra_pdfidf'])
        if ('certificates' in self
                or 'ssh_access' in self
                or 'ssh_creds' in self):
            self._import_certs()

    def _import_certs(self):
        ini = getConfig('ini')
        if 'certificates' in self:
            # pylint: disable=unused-variable
            (artifact_type, base, final) = self._get_source(
                extension='',
                folder=ini['artifacts']['certificates']['src'])
            log.debug("Artifact[%s]: load 'certificates'", self.step_name)
            self._add_source(
                {'type': artifact_type,
                 'destination': ini['artifacts']['certificates']['dest'],
                 'source': f"{base}/{self['certificates']}"})
        if 'ssh_creds' in self:
            # pylint: disable=unused-variable
            (artifact_type, base, final) = self._get_source(
                extension='',
                folder=ini['artifacts']['ssh_creds']['src'])
            log.debug("Artifact[%s]: load 'ssh_creds'", self.step_name)
            self._add_source(
                {'type': artifact_type,
                 'destination': ini['artifacts']['ssh_creds']['dest'],
                 'source': f"{base}/{self['ssh_creds']}"})
        if 'ssh_access' in self:
            # pylint: disable=unused-variable
            (artifact_type, base, final) = self._get_source(
                extension='',
                folder=ini['artifacts']['ssh_access']['src'])
            log.debug("Artifact[%s]: load 'ssh_access'", self.step_name)
            self._add_source(
                {'type': artifact_type,
                 'destination': ini['artifacts']['ssh_access']['dest'],
                 'source': f"{base}/{self['ssh_access']}"})

    def _import_infra_pdfidf(self, infra):
        """Import PDF/IDF."""
        ini = getConfig('ini')
        (artifact_type, base, final) = self._get_source(
            extension=ini['artifacts']['pdfidf']['extension'],
            folder=ini['artifacts']['pdfidf']['src'])
        idf_prefix = ini['artifacts']['pdfidf']['idf_prefix']
        self._add_source({
            'type': artifact_type,
            'source': f"{base}/{infra}{final}",
            'destination': ini['artifacts']['pdfidf']['pdf_dest']})
        self._add_source({
            'type': artifact_type,
            'source': f"{base}/{idf_prefix}{infra}{final}",
            'destination': ini['artifacts']['pdfidf']['idf_dest']})

    def _get_source(self, extension, folder):
        """Get source parameters from config_src.

        for 'static_src' and 'infra_pdfidf' source can be remote from url, api
        or can be local.
        """
        parser = JinjaParser()
        branch = parser.parse(self.config_source['branch'])
        path = (parser.parse(self.config_source['path'])+f'/{folder}'
                if 'path' in self.config_source else folder)
        # in case of remote config
        if 'api' in self.config_source:
            artifact_type = "remote"
            base = (f"{ self.config_source['api'] }/repository/files/"
                    f"{ urllib.parse.quote(path) }")
            final = f"{extension}?ref={ branch }"
        elif 'url' in self.config_source:
            artifact_type = "remote"
            base = (f"{ self.config_source['url'] }/raw/{ branch }/"
                    f"{ urllib.parse.quote(path) }")
            final = extension
        else:
            artifact_type = "local"
            base = f"{ path }"
            final = extension
        return(artifact_type, base, final)

    @property
    def bin(self):
        """Get the base64 encoded binary value of the artifact."""
        log.info("Artifact[%s]: generate bin", self.step_name)
        self._load_artifactfolder()
        self._encrypt_artifactfolder()
        content = self._zip_artifactfolder()
        self._clean_artifactfolder()
        return base64.b64encode(content)

    def get_local(self, dest='./'):
        """Copy the artifact file to a specific folder."""
        log.info("Artifact[%s]: get artifacts to '%s'", self.step_name, dest)
        self._load_artifactfolder()
        self._encrypt_artifactfolder()
        copy_tree(self.folder, dest)
        self._clean_artifactfolder()

    def _zip_artifactfolder(self):
        """Zip the artifact folder."""
        log.debug("Artifact[%s]: zip artifact folder", self.step_name)
        # pylint: disable=R1732
        tmp_file = NamedTemporaryFile(suffix='.zip')
        zip_name = tmp_file.name.replace('.zip', '')
        # Archive the folder
        zip_file = make_archive(zip_name,
                                'zip',
                                root_dir=self.folder)
        # Get the binary content of the file
        with open(zip_file, mode='rb') as file:
            content = file.read()
        tmp_file.close()
        return content

    def _clean_artifactfolder(self):
        """Clean artifact folder."""
        rmtree(self.folder)

    def _load_artifactfolder(self):
        """Load all the artifactSrc."""
        log.debug("Artifact[%s]: load artifact folder", self.step_name)
        # Create temp folder
        self.folder = mkdtemp()
        # Get all sources to this folder
        for artifact_src in self.sources:
            artifact_src.load(self.folder)

    def _encrypt_artifactfolder(self):
        """Encrypt all files of the temporary folder."""
        ini = getConfig('ini')
        if self['encrypt']:
            log.debug("Artifact[%s]: encrypt artifact archive", self.step_name)
            if ini['encryption']['method'] == 'ansible_vault':
                self._load_vault_lib()
            else:
                raise ValueError(
                    "Config encryption:method' must be in "
                    f"'{['ansible_vault']}' not "
                    f"'{ini['encryption']['method']}'")
            # pylint: disable=unused-variable
            for root, dirs, files in walk(self.folder):
                for file in files:
                    self._encrypt_artifact_file(f'{root}/{file}')

    def _encrypt_artifact_file(self, filepath):
        """Encrypt on file of the temporary folder."""
        ini = getConfig('ini')
        if ini['encryption']['method'] == 'ansible_vault':
            self._encrypt_artifact_file_vault(filepath)

    def _load_vault_lib(self):
        """Load VaultLib to encypt files."""
        ini = getConfig('ini')
        run = getConfig('run')
        if not run.get('key'):
            raise ValueError(
                f"'{ini['encryption']['key_env_name']}' variable must be set")
        self.vault_ed = VaultEditor()

    def _encrypt_artifact_file_vault(self, filepath):
        """Encrypt file with ansible vault."""
        run = getConfig('run')
        log.debug("Artifact[%s]: vault file '%s'", self.step_name, filepath)
        vault_key = run.get('key')
        try:
            self.vault_ed.encrypt_file(filepath, VaultSecret(vault_key))
        except AnsibleError as excpt:
            if f"{excpt}" == "input is already encrypted":
                log.info("%s is already encrypted, continue", filepath)
            else:
                raise excpt
        except Exception as excpt:
            log.error("Unexpected error")
            raise excpt

    def _add_source(self, src):
        """Add a source to the artifact."""
        self.sources.append(ArtifactSrc(src))

    def _validate(self, src):
        """Validate the artifact config."""
        log.debug("Artifact[%s]: validation", self.step_name)
        schema = Schema({
            Optional('get_artifacts'): Or(str, [dict]),
            Optional('local_files'): [dict],
            Optional('remote_files'): [dict],
            Optional('certificates'): str,
            Optional('ssh_creds'): str,
            Optional('ssh_access'): str,
            Optional('infra_pdfidf'): str,
            'encrypt': bool})
        return schema.validate(dict(src))
